"use strict";

exports.__esModule = true;
exports.default = Logger;

var _ora = _interopRequireDefault(require("ora"));

var _chalk = _interopRequireDefault(require("chalk"));

var _guid = _interopRequireDefault(require("guid"));

var _crypto = _interopRequireDefault(require("crypto"));

var _server = _interopRequireDefault(require("../server"));

var _types = require("../store/types");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

let running = false;
let spin = false;
let spinner = (0, _ora.default)('');

function createColorHash(name, index = 0) {
  let hash = name ? _crypto.default.createHash('md5').update(name).digest('hex').slice(0, 6) : '000000';
  let array = hash.split('');
  array[index + 0] = 'f';
  array[index + 1] = 'f';
  return '#' + array.join('');
}

function Logger(config = {}) {
  config = _extends({
    characterLimit: 80,
    collapseErrors: false,
    spinOnFirstUser: true,
    filter: ['task'],
    removeFromStore: true,
    moreColors: true
  }, config);
  return store => next => action => {
    if (action.type === _types.updateGlobals && action.running !== undefined) {
      running = action.running;
      if (!config.spinOnFirstUser && running) spin = true;
    } else if (action.type === _types.log && (!config.filter || !config.filter.includes(action.args.owner))) {
      if (config.spinOnFirstUser && action.args.owner === 'user' && running) spin = true;
      let {
        level,
        text
      } = action;
      let {
        owner,
        user,
        worker,
        task,
        duration
      } = action.args;
      let colors = {
        system: 'bgBlue',
        user: 'blue',
        worker: 'white',
        task: 'cyan'
      };
      let receiving = text.startsWith('receiv');
      let separating = text.startsWith('-');
      owner = owner.slice(0, 6); //$FlowFixMe

      owner = _chalk.default[config.moreColors && colors[owner] || 'white'](new Array(6 - owner.length).fill(' ').join('') + owner.toUpperCase());
      worker = _chalk.default.hex(createColorHash(worker, 0))(worker ? '◼' : ' ');
      user = _chalk.default.hex(createColorHash(user, 2))(user ? '◼' : ' ');
      task = _chalk.default.hex(createColorHash(task, 4))(task ? '◼' : ' ');

      if (action.args.owner === 'system' && separating) {
        worker = _chalk.default.white('◼ ') + _chalk.default.blue('◼ ') + _chalk.default.cyan('◼ ') + _chalk.default.white('Worker');
        user = _chalk.default.blue('User');
        task = _chalk.default.cyan('Task');
        text = '';
      }

      if (level !== 'error' || config.collapseErrors) {
        if (duration) text = `${duration.toFixed(2)}ms ${text}`;
        text = text.slice(0, config.characterLimit);
        if (receiving) text = _chalk.default.dim(text);
      }

      text = `${owner} ${worker} ${user} ${task} ${text}`;
      if (level === 'info' && text.includes('accept')) spinner.succeed(text);else if (level === 'info') spinner.stopAndPersist({
        text: config.moreColors ? _chalk.default.cyan(text) : text
      });else if (level === 'log') spinner.stopAndPersist({
        text
      });else if (level === 'warn') spinner.warn(_chalk.default.yellow(text));else if (level === 'error') spinner.fail(_chalk.default.red(text));
      running && spin && spinner.start();
    }

    if (action.type === _types.log && action.level !== 'error' && config.removeFromStore) return;
    return next(action);
  };
}