"use strict";

exports.__esModule = true;
exports.default = void 0;

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

const path = require('path');

const os = require('os');

const fs = require('fs');

const spawn = require('threads').spawn;

class Server {
  constructor() {
    this.guid = 0;
  }

  start(_ref) {
    let {
      binary,
      middleware
    } = _ref,
        config = _objectWithoutPropertiesLoose(_ref, ["binary", "middleware"]);

    this.messages = {};
    this.thread = spawn(async ([args], send) => {
      try {
        let results = undefined;

        switch (args.task) {
          case 'start':
            {
              const Server = require(args.binary).default;

              this.instance = new Server(args.config, args.middleware && require(args.middleware.binary).default(args.middleware.config)); // Start local CC server

              await this.instance.start();
              break;
            }

          case 'stop':
            {
              await this.instance.stop(args.force);
              break;
            }

          case 'flush':
            {
              await this.instance.flush(args.force);
              break;
            }

          case 'addUser':
            {
              this.instance.actions.addUser(args.user);
              break;
            }

          case 'removeUser':
            {
              this.instance.actions.removeUser(args.user);
              break;
            }

          case 'createTask':
            {
              const response = await this.instance.actions.createTask(args.command, args.user);
              results = Object.assign(response, {
                promise: undefined,
                reject: undefined,
                resolve: undefined
              });
              break;
            }

          case 'subscribe':
            {
              eval('var fn = ' + args.selector);
              this.instance.subscribe(fn, results => send({
                id: args.id,
                results
              }));
              break;
            }
        }

        return {
          id: args.id,
          results
        };
      } catch (e) {
        console.error(e);
      }
    });
    this.thread.on('message', response => {
      if (response && this.messages[response.id]) {
        const {
          id,
          results
        } = response;
        const {
          resolve,
          callback
        } = this.messages[id];

        if (callback) {
          if (results === undefined) {
            resolve(results);
          } else {
            callback(results);
          }
        } else {
          resolve(results);
          delete this.messages[id];
        }
      }
    });
    return this._task('start', _extends({
      binary,
      middleware
    }, config));
  }

  async stop(force = false) {
    await this._task('stop', {
      force
    });
    this.thread.kill();
  }

  flush(force = false) {
    return this._task('flush', {
      force
    });
  }

  addUser(user) {
    return this._task('addUser', {
      user
    });
  }

  removeUser(user) {
    return this._task('removeUser', {
      user
    });
  }

  createTask(command, user) {
    return this._task('createTask', {
      command,
      user
    });
  }

  subscribe(selector, callback) {
    return this._task('subscribe', {
      selector
    }, callback);
  }

  _task(task, args, callback) {
    const id = this.guid++;
    let resolve = undefined;
    let promise = new Promise(res => resolve = res);
    this.messages[id] = {
      resolve,
      callback
    };
    this.thread.send([_extends({
      task
    }, args, {
      id,
      selector: args.selector && args.selector.toString()
    })]);
    return promise;
  }

}

exports.default = Server;