"use strict";

exports.__esModule = true;
exports.default = void 0;

var _redux = require("redux");

var _reduxThunk = _interopRequireDefault(require("redux-thunk"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _path = _interopRequireDefault(require("path"));

var _osUtils = _interopRequireDefault(require("os-utils"));

var _stoppable = _interopRequireDefault(require("stoppable"));

var _express = _interopRequireDefault(require("express"));

var _connectTimeout = _interopRequireDefault(require("connect-timeout"));

var _serveIndex = _interopRequireDefault(require("serve-index"));

var _guid = _interopRequireDefault(require("guid"));

var _portfinder = _interopRequireDefault(require("portfinder"));

var _isPortAvailable = _interopRequireDefault(require("is-port-available"));

var _stringifyObjectExtractFunctions = _interopRequireDefault(require("stringify-object-extract-functions"));

var _fixPath = _interopRequireDefault(require("fix-path"));

var _helpers = require("./helpers");

var actions = _interopRequireWildcard(require("./store/actions"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class Server {
  constructor(config, middleware = []) {
    _defineProperty(this, "store", void 0);

    _defineProperty(this, "actions", void 0);

    _defineProperty(this, "express", void 0);

    _defineProperty(this, "socket", void 0);

    _defineProperty(this, "io", void 0);

    _defineProperty(this, "ws", void 0);

    _defineProperty(this, "getStore", () => this.store);

    _defineProperty(this, "getState", () => this.store.getState());

    _defineProperty(this, "subscribe", (selector, callback) => (0, _helpers.subscribe)(this.store, selector, callback));

    _defineProperty(this, "awaitState", async (selector, condition, timeout) => await (0, _helpers.awaitState)(this.store, selector, condition, timeout));

    _defineProperty(this, "health", () => 200);

    try {
      // workaround for: https://github.com/electron/electron/issues/7688
      (0, _fixPath.default)(); // Create redux store

      this.store = (0, _redux.createStore)((0, _redux.combineReducers)({
        config: require('./store/reducers/config').default,
        users: require('./store/reducers/users').default,
        workers: require('./store/reducers/workers').default,
        tasks: require('./store/reducers/tasks').default,
        collections: require('./store/reducers/collections').default,
        globals: require('./store/reducers/globals').default
      }), (0, _redux.applyMiddleware)(_reduxThunk.default.withExtraArgument(this), ...(Array.isArray(middleware) ? middleware : [middleware]))); // 1. Bind actions and start observing the store

      this.actions = (0, _redux.bindActionCreators)(actions, this.store.dispatch); // 2. Watch task queue ...

      this.subscribe(state => ({
        tasks: state.tasks,
        workers: state.workers
      }), this.actions.handleTasks); // 3. Override default server config

      if (config) this.actions.updateConfig(config); // 4. Check and create folders

      const output = this.getState().config.output;

      const statepath = _path.default.join(output, '/states');

      if (_fsExtra.default.existsSync(statepath)) _fsExtra.default.removeSync(statepath);

      _fsExtra.default.mkdirSync(statepath);

      const logpath = _path.default.join(output, '/LogFiles');

      !_fsExtra.default.existsSync(logpath) && _fsExtra.default.mkdirSync(logpath); // Delete logs older than 5 days

      _fsExtra.default.readdirSync(logpath).map(name => ({
        name,
        time: _fsExtra.default.statSync(_path.default.join(logpath, name)).mtime.getTime() * 1000 * 60 * 60 * 24
      })).filter(a => a.time > this.getState().config.logExpirationDays).forEach(a => _fsExtra.default.unlinkSync(_path.default.join(logpath, a.name)));
    } catch (error) {
      console.error(error);
    }
  }

  async start() {
    try {
      let {
        protocol,
        serverOptions,
        output,
        privatePort,
        publicPort,
        ip,
        instances,
        restTimeout,
        pingTimeout,
        pingInterval
      } = this.getState().config;
      this.actions.info('initializing ...');
      this.actions.cleanup(); // Stage 1: occupy random port if none was given explicitely

      if (!privatePort || (await (0, _isPortAvailable.default)(privatePort)) === false) {
        _portfinder.default.basePort = 9000 + parseInt(Math.random() * 1000);
        privatePort = await _portfinder.default.getPortPromise();
        this.actions.updateConfig({
          privatePort
        });
      } // Stage 2: prepare sockets


      this.express = (0, _express.default)();
      this.express.use(require('compression')());
      this.express.use(require('cors')({
        credentials: true,
        origin: true
      }));
      this.express.use(_express.default.json({
        limit: '10mb'
      }));
      this.express.use(_express.default.urlencoded({
        limit: '10mb',
        extended: true
      }));
      this.socket = (0, _stoppable.default)(protocol === 'https' ? require('https').createServer(serverOptions, this.express) : require('http').createServer(this.express));
      this.socket.timeout = 0;
      this.socket.keepAliveTimeout = 0;
      this.socket.maxConnections = 100000;
      this.io = require('socket.io')(this.socket, {
        serveClient: false,
        pingTimeout,
        pingInterval
      });
      this.socket.listen(publicPort, ip);
      this.socket.on('error', err => console.log(err)); // Stage 3: listen ...

      this.actions.info(`accept ${protocol}://${ip}:${publicPort}/`, {
        owner: 'io'
      });
      this.io.on('connection', this.actions.handleUser); // REST health endpoint, used for load balancer probing

      this.actions.info(`accept ${protocol}://${ip}:${publicPort}/health`, {
        owner: protocol
      });
      this.express.get('/health', (req, res) => res.sendStatus(this.health())); // REST Status endpoint

      this.actions.info(`accept ${protocol}://${ip}:${publicPort}/status`, {
        owner: protocol
      });
      this.express.get('/status', (req, res) => {
        const currentState = this.getState();

        const state = _extends({}, currentState, {
          config: _extends({}, currentState.config)
        });

        delete state.config.executable;
        delete state.config.output;
        delete state.config.exec;
        delete state.config.execArgs;
        res.type('text/plain').send((0, _stringifyObjectExtractFunctions.default)(state, {
          indent: '  ',
          inlineCharacterLimit: 256
        }));
      }); // REST Status endpoint

      const logPath = this.getState().config.output + '/LogFiles';
      this.actions.info(`accept ${protocol}://${ip}:${publicPort}/log`, {
        owner: protocol
      });
      this.express.use('/log', _express.default.static(logPath), (0, _serveIndex.default)(logPath, {
        icons: true,
        view: 'details'
      })); // REST Login

      this.actions.info(`accept ${protocol}://${ip}:${publicPort}/login`, {
        owner: protocol
      });
      this.express.get('/login', (req, res) => {
        req.socket.setTimeout(restTimeout);

        const id = _guid.default.raw();

        this.actions.addUser(id);
        res.json({
          op: 'system',
          status: 'permitted',
          id
        });
      }); // REST Logout

      this.actions.info(`accept ${protocol}://${ip}:${publicPort}/logout/:id`, {
        owner: protocol
      });
      this.express.get('/logout/:id', (req, res) => {
        req.socket.setTimeout(restTimeout);
        this.actions.removeUser(req.params.id);
        res.type('text/plain').send(`disconnected`);
      }); // REST POST Execute

      this.actions.info(`accept ${protocol}://${ip}:${publicPort}/execute/:id`, {
        owner: protocol
      });
      this.express.post('/execute/:id', async (req, res) => {
        req.socket.setKeepAlive(true);

        try {
          let context = await this.actions.createTask(req.body, req.params.id);
          res.json(context.results);
        } catch (reason) {
          this.actions.error(reason);
          res.status(500).json({
            message: reason.message
          });
        }
      }); // Internal WebSocket queue (used only by workers)...

      this.actions.info(`accept ws://${ip}:${privatePort}/`, {
        owner: 'ws'
      });
      this.ws = new (require('ws').Server)({
        port: privatePort
      });
      this.ws.on('connection', this.actions.handleWorker);
      this.ws.on('error', this.actions.error); // Stage 4: signal the running state

      this.store.dispatch(actions.updateGlobals({
        running: true
      })); // Stage 5: spawn local workers

      this.actions.info(new Array(1).fill('-').join(''));
      if (instances > 0) await Promise.all(new Array(instances).fill().map(() => this.actions.spawnWorker()));
    } catch (error) {
      this.actions.error(error);
      return error;
    }

    return this;
  }

  async flush(force = false) {
    try {
      this.actions.info('flushing workers ...');
      await this.stop(force);
      await this.start();
    } catch (error) {
      console.error(error);
    }
  }

  async stop(force = false) {
    try {
      this.actions.info('shutting down ...');
      this.actions.updateGlobals({
        running: false
      });
      await this.actions.cleanup(force);
      this.io && this.io.close();
      this.ws && this.ws.close();
      this.socket && this.socket.stop();
      this.socket = this.io = this.ws = this.express = undefined;
    } catch (error) {
      console.error(error);
    }
  }

}

exports.default = Server;