"use strict";

exports.__esModule = true;
exports.cleanup = cleanup;
exports.updateGlobals = updateGlobals;
exports.updateConfig = updateConfig;
exports.error = error;
exports.info = info;
exports.log = log;
exports.warn = warn;
exports.debug = debug;

var types = _interopRequireWildcard(require("../types"));

var workerActions = _interopRequireWildcard(require("./worker"));

var userActions = _interopRequireWildcard(require("./user"));

var taskActions = _interopRequireWildcard(require("./task"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function cleanup(force = false) {
  return async (dispatch, getState) => {
    const {
      collections,
      workers,
      users,
      tasks
    } = getState();
    await Promise.all(collections.workers.map(id => dispatch(workerActions.removeWorker(id, force))));
    collections.users.forEach(id => dispatch(userActions.removeUser(id)));
    tasks.forEach(task => dispatch(taskActions.removeTask(task.id)));
    dispatch(updateGlobals({
      log: []
    }));
  };
}

function updateGlobals(props) {
  return _extends({
    type: types.updateGlobals
  }, props);
}

function updateConfig(props) {
  return _extends({
    type: types.updateConfig
  }, props);
}

function error(reason, args) {
  return {
    type: types.log,
    level: 'error',
    text: reason.stack || reason,
    args: _extends({
      owner: 'system'
    }, args)
  };
}

function info(text, args) {
  return {
    type: types.log,
    level: 'info',
    text,
    args: _extends({
      owner: 'system'
    }, args)
  };
}

function log(text, args) {
  return {
    type: types.log,
    level: 'log',
    text,
    args: _extends({
      owner: 'system'
    }, args)
  };
}

function warn(text, args) {
  return {
    type: types.log,
    level: 'warn',
    text,
    args: _extends({
      owner: 'system'
    }, args)
  };
}

function debug(text, args) {
  return {
    type: types.log,
    level: 'debug',
    text,
    args: _extends({
      owner: 'system'
    }, args)
  };
}